@@ -3,7 +3,8 @@
 //
 //  This file is part of Linterna Mágica
 //
-//  Copyright (C) 2010, 2011, 2012  Ivaylo Valkov <ivaylo@e-valkov.org>
+//  Copyright (C) 2010, 2011, 2012 Ivaylo Valkov <ivaylo@e-valkov.org>
+//  Copyright (C) 2013, 2014 Ivaylo Valkov <ivaylo@e-valkov.org>
 //  Copyright (C) 2010  Anton Katsarov <anton@katsarov.org>
 //
 //  The JavaScript code in this page (or file) is free software: you
@@ -23,7 +24,7 @@
 //  @licend The above is the entire license notice for the JavaScript
 //  code in this page (or file).
 //
-// @source http://linterna-magica.nongnu.org
+// @source http://linternamagica.org
 
 // END OF LICENSE HEADER
 
@@ -176,6 +177,14 @@ LinternaMagica.prototype.create_youtube_links = function(fmt, fmt_url_map)
 
 	    link.url = fmt_url_map[fmt_id];
 
+	    if (/https:/i.test(window.location.protocol) &&
+		/http:/i.test(link.url))
+	    {
+		link.url = link.url.replace("http:", "https:");
+		this.mixed_content = true;
+	    }
+
+
 	    this.log("LinternaMagica.create_youtube_links:\n"+
 		     "Extracted link  : "+link.url,4);
 
@@ -188,53 +197,6 @@ LinternaMagica.prototype.create_youtube_links = function(fmt, fmt_url_map)
     return null;
 }
 
-// Detect youtube flash upgrade warning. This is called withing
-// setInterval. It is needed because when the elements with the
-// warning are inserted all our data that has been added before that
-// is removed.
-LinternaMagica.prototype.detect_youtube_flash_upgrade = function(object_data)
-{
-    this.youtube_flash_upgrade_counter++;
-
-    // Fancy flash upgrade message. The element with id flash-upgrade
-    // might be obsolete now.
-    var watch_player = document.getElementById("watch7-player");
-    var fancy_alert = null;
-
-    if(watch_player && watch_player.hasAttribute("class") &&
-       /flash-player/i.test(watch_player.getAttribute("class")))
-    {
-	var alert = watch_player.querySelector(".yt-alert-message");
-
-	if (alert && /flash player/i.test(alert.textContent))
-	{
-	    fancy_alert = true;
-	}
-	
-    }
-    
-
-    // With default timeout 3000mS this will be >10 sec. Stop checking and insert.    
-    // Might be flashblock
-    if (document.getElementById("movie_player") ||
-	document.getElementById("movie_player-html5") ||
-	fancy_alert ||
-	this.youtube_flash_upgrade_counter >= 5 )
-    {
-	clearInterval(this.youtube_flash_upgrade_timeout);
-
-	this.log("LinternaMagica.detect_youtube_flash_upgrade:\n"+
-		 "Removing plugin install warning.",2);
-
-	this.remove_plugin_install_warning(object_data.parent);
-
-	this.log("LinternaMagica.detect_youtube_flash_upgrade:\n"+
-		 "Creating video object.",2);
-
-	setTimeout(this.create_video_object(object_data), 1000);
-    }
-}
-
 // Extract links data for youtube from fmt_url_map
 LinternaMagica.prototype.extract_youtube_fmt_url_map = function()
 {
@@ -270,7 +232,15 @@ LinternaMagica.prototype.extract_youtube_fmt_url_map = function()
 	    var link = fmt[url].match(/(url|conn)=([^&]+)/);
 	    var fmt_id = fmt[url].match(/itag=([0-9]+)/);
 	    var sig = fmt[url].replace(/\\u0026/g, '&').match(/sig=[^&]+/);
-		sig = sig[sig.length-1].replace(/sig/,'signature');
+	    sig = sig ? sig[sig.length-1].replace(/sig/,'signature') : '';
+
+	    // Bug 39402. Some links have the signature in the s=<SIG>
+	    // format. For example when they are oppened from Canada
+	    if (!sig)
+	    {
+		this.sites["youtube.com"].encrypted_signature = 1;
+		sig = "";
+	    }
 	    
 	    if (fmt_id && link)
 	    {
@@ -304,7 +274,7 @@ LinternaMagica.prototype.sites["www.youtube.com"] = "youtube.com";
 LinternaMagica.prototype.sites["www.youtube-nocookie.com"] = "youtube.com";
 LinternaMagica.prototype.sites["youtube-nocookie.com"] = "youtube.com";
 
-// Referenced by Vimeo
+// Referenced by Vimeo, Xhamster
 LinternaMagica.prototype.sites["youtube.com"].flash_plugin_installed =
 function()
 {
@@ -319,6 +289,32 @@ function()
 LinternaMagica.prototype.sites["youtube.com"].skip_script_processing =
 function()
 {
+
+    if (!this.fixed_youtube_navigation)
+    {
+	this.fixed_youtube_navigation = 1;
+
+	// Fix (for LM) YouTube's awful AJAX navigation.
+	this.fix_youtube_navigation();
+
+	// Check for unavailable clips in playlists and switch to the next
+	// clip.  YT is using some strange loading technique that looks
+	// like AJAX/XMLHttpRequest, which breaks LM loading. Leaving this
+	// task to YT is not an option.
+
+	var unavailable = document.getElementById("player-unavailable");
+	if (unavailable &&
+	    (!this.object_has_css_class(unavailable, "hid") ||
+	     getComputedStyle(unavailable).getPropertyValue("display") != "none"))
+	{
+	    var object_data = new Object();
+	    object_data.linterna_magica_id = null;
+	    this.sites["youtube.com"].css_fixes.apply(this, [object_data]);
+	    this.sites["youtube.com"].player_stream_ended_action.apply(this,[]);
+	    return;
+	}
+    }
+
     // See bug #108013:
     // https://savannah.nongnu.org/support/index.php?108013
     // Do not skip script processing in youtube. The default function
@@ -352,8 +348,9 @@ LinternaMagica.prototype.sites["youtube.com"].skip_video_id_extraction = functio
 LinternaMagica.prototype.sites["youtube.com"].extract_object_from_script =
 function()
 {
+
     var data = this.script_data;
-    if (!data.match(/yt\.playerConfig =/))
+    if (!data.match(/ytplayer\.config =/))
     {
  	return null;
     }
@@ -369,7 +366,7 @@ function()
     var hd_links = this.create_youtube_links(fmt, maps);
     var link = (hd_links && hd_links.length) ? hd_links : null;
 
-    var embed_id = data.match(/\"id\"\:\s*\"([a-zA-Z0-9_\-]+)\"/);
+    var player_id = data.match(/\"id\"\:\s*\"([a-zA-Z0-9_\-]+)\"/);
 
     // We do not have any links! Give up.
     if (!link)
@@ -387,19 +384,37 @@ function()
 	width = width[width.length-1];
     }
 
-    if (embed_id)
+    if (player_id)
     {
-	embed_id= embed_id[embed_id.length-1];
+	player_id = player_id[player_id.length-1];
+    }
+    else
+    {
+	player_id = "movie_player";
     }
 
-    var p = this.get_first_element_by_class("flash-player");
-    p = p ? p : this.get_first_element_by_class("html5-player");
-
+    var player = document.getElementById(player_id);
+    var player_api = document.getElementById("player-api");
 
     if (!width || !height)
     {
-	height = p ? p.clientHeight : null;
-	width  = p ? p.clientWidth : null;
+	height = player ? player.clientHeight : null;
+	width  = player ? player.clientWidth : null;
+    }
+
+    if (!width || !height)
+    {
+	height = ( player && player.parentNode ) ?
+	    player.parentNode.clientHeight : null;
+	width  = ( player && player.parentNode ) ?
+	    player.parentNode.parentNode.clientWidth : null;
+    }
+
+    if (!width || !height)
+    {
+	var cs = window.getComputedStyle(player_api);
+	width = parseInt(cs.getPropertyValue("width"));
+	height = parseInt(cs.getPropertyValue("height"));
     }
 
     if (!width || !height)
@@ -407,7 +422,7 @@ function()
 	this.log("LinternaMagica.extract_object_from_script_youtube:\n"+
 		 "Missing object data "+
 		 "\n H: "+height+
-		 "\n W:"+width, 3);
+		 "\n W: "+width, 3);
 
 	return null;
     }
@@ -415,50 +430,35 @@ function()
     this.log("LinternaMagica.extract_object_from_script_youtube:\n"+
 	     " H: "+height+
 	     "\n W:"+width+
-	     "\n embedid "+embed_id,2);
+	     "\n player/swf "+player_id,2);
 
 
     var object_data = new Object();
-    var linterna_magica_id = null;
 
     object_data.width= width;
     object_data.height= height;
     object_data.link = hd_links ? hd_links[hd_links.length-1].url : null;
     object_data.hd_links = link;
+    object_data.parent = player_api.parentNode;
+    object_data.site_player = player_api;
 
-    if (embed_id)
+    if (this.is_swf_object(player))
     {
-	embed_object = document.getElementById(embed_id);
-	if (embed_object)
-	{
-	    if (this.plugin_is_installed)
-	    {
-		linterna_magica_id =
-		    this.mark_flash_object(embed_object);
-
-		object_data.parent = embed_object.parentNode;
-	    }
-	}
+	this.priority = true;
+	object_data.linterna_magica_id =
+	    this.mark_flash_object(player);
     }
-
-    if (!embed_id ||
-	!embed_object ||
-	(embed_object &&
-	 !this.plugin_is_installed))
+    else
     {
-	linterna_magica_id =
+	object_data.linterna_magica_id =
 	    this.mark_flash_object("extracted-from-script");
-
-	object_data.parent =  p;
     }
 
-    object_data.linterna_magica_id = linterna_magica_id;
-
     return object_data;
 }
 
 LinternaMagica.prototype.sites["youtube.com"].
-    stop_if_one_extracted_object_from_script =
+stop_if_one_extracted_object_from_script =
 function()
 {
     this.log("LinternaMagica.sites.stop_if_"+
@@ -468,51 +468,207 @@ function()
 }
 
 LinternaMagica.prototype.sites["youtube.com"].
-    replace_extracted_object_from_script = 
+replace_extracted_object_from_script =
 function(object_data)
 {
-    // This function used to exit if flash plugin was installed,
-    // because no workarounds were needed. Now Epiphany requires this
-    // delay, because it prevents LM to be inserted before the HTML5
-    // player, in which case they are both playing and overlapping.
-
-    if (!this.youtube_flash_upgrade_timeout)
-    {
-	this.youtube_flash_upgrade_counter = 0;
-	var data = object_data;
-	var self = this;
-
-	this.youtube_flash_upgrade_timeout = setInterval(
-	    function() {
-		self.detect_youtube_flash_upgrade.apply(self,[data]);
-	    }, 3000);
-    }
-    
+    this.create_video_object(object_data);
     return false;
 }
 
+LinternaMagica.prototype.sites["youtube.com"].player_stream_ended_action =
+function()
+{
+    var playlist = document.getElementById("watch-appbar-playlist");
+    var autoplay =
+	this.get_first_element_by_class("toggle-autoplay", playlist);
+
+    var current_song =
+	this.get_first_element_by_class("currently-playing", playlist);
+
+    var shuffle =
+	this.get_first_element_by_class("shuffle-playlist", playlist);
+
+    var next_song = null;
+
+    if (!playlist || !autoplay || !current_song ||
+	!this.object_has_css_class(autoplay, "yt-uix-button-toggled"))
+    {
+	return;
+    }
+
+    if (shuffle && 
+	this.object_has_css_class(shuffle, "yt-uix-button-toggled"))
+    {
+	var songs = playlist.getElementsByTagName("a");
+	var rand = Math.floor(Math.random() * songs.length);
+	next_song = songs[rand].getAttribute("href");
+    }
+    else 
+    {
+	if (/text/i.test(current_song.nextSibling))
+	{
+	    next_song = current_song.nextSibling.nextSibling;
+	}
+	else
+	{
+	    next_song = current_song.nextSibling;
+	}
+
+	next_song = 
+	    next_song.getElementsByTagName("a")[0].
+	    getAttribute("href");
+    }
+
+    window.location.href = next_song;
+
+    return true;
+}
+
+LinternaMagica.prototype.fix_youtube_navigation = function()
+{
+    // Fix history navigation (HTML5) for LM
+    window.addEventListener("popstate", function() {
+	setTimeout(function() { window.location.reload(); }, 50);
+    }, true);
+
+    // Fix search box
+    var search_btn = document.getElementById("search-btn");
+    var search = document.getElementById("masthead-search-term");
+    var form = document.getElementById("masthead-search");
+    var submit_form_on_enter_fn =  function(ev)
+    {
+	var key = ev.keyCode || ev.charCode;
+	if (key == 13)
+	{
+	    ev.preventDefault();
+	    var form =
+		document.getElementById("masthead-search");
+	    form.submit();
+	}
+    };
+
+    var submit_form_on_click_fn = function(e)
+    {
+	var form =
+	    document.getElementById("masthead-search");
+	form.submit();
+    };
+
+    if (form)
+    {
+	form.removeAttribute("onsubmit");
+    }
+
+    if (search)
+    {
+	search.addEventListener("keydown", submit_form_on_enter_fn, false);
+    }
+
+    if (search_btn)
+    {
+	search_btn.addEventListener("click", submit_form_on_click_fn);
+    }
+
+    // Override YT links click events. Ensure LM will load when a new
+    // link is selected.
+    var links = document.getElementsByTagName("a");
+
+    // Disabled for testing
+    // links = null;
+    if (links && links.length > 0)
+    {
+	var clip_click_fn = function(ev)
+	{
+	    var url = this.href;
+
+	    if (!url)
+	    {
+		var a = this.getElementsByTagName('a')[0];
+		url = a.getAttribute('href');
+	    }
+
+	    if (!url)
+	    {
+		return;
+	    }
+
+	    setTimeout(function()
+		       {
+			   window.location.href = url;
+		       },50);
+	};
+
+	for (var i=0,l=links.length; i<l;i++)
+	{
+	    var t = links[i];
+	    var href = t.getAttribute("href");
+	    var id =  t.getAttribute("id");
+	    var skip_re = new RegExp ( "data:|javascript:|#");
+
+	    if (id && id == "logo-container")
+	    {
+		continue;
+	    }
+
+	    if (href && href.match(skip_re))
+	    {
+		continue;
+	    }
+
+	    t.addEventListener('click',clip_click_fn,false);
+	}
+    }
+}
+
 LinternaMagica.prototype.sites["youtube.com"].css_fixes =
 function(object_data)
 {
-
-    // Sometimes when flash is installed the flash video object does
-    // not have (at all or the right one) linterna_magica_id. Usually
-    // the other objects if any are useless. This renders both LM and
-    // the flash interface. The code bellow tries to avoid it. Reason
-    // *unknown*.
-
-    this.log("LinternaMagica.youtube.css_fixes:\n "+
-	     "Harvesting (possible) lost flash video object with "+
-	     "linterna_magica_id "+ object_data.linterna_magica_id);
-
-    var movie_player = document.getElementById('movie_player');
-    if (movie_player) {
-	movie_player.linterna_magica_id = object_data.linterna_magica_id;
+    var id = object_data.linterna_magica_id;
+    var toggle_plugin = document.getElementById("linterna-magica-toggle-plugin-"+id);
+    if (toggle_plugin)
+    {
+	toggle_plugin = toggle_plugin.parentNode.parentNode;
+	toggle_plugin.style.setProperty("width", object_data.width+"px", "important");
     }
 
-    if (this.priority.self > this.priority.plugin)
+    var playlist = document.getElementById("watch7-playlist-tray-container");
+    if (playlist)
     {
-	this.hide_flash_video_object(object_data.linterna_magica_id);
+	var lm = document.getElementById("linterna-magica-"+id);
+	if (lm)
+	{
+	    lm.style.setProperty("float", "left", "important");
+	}
+    }
+
+
+    var player_api  = document.getElementById("player-api");
+    if (this.priority)
+    {
+	if (player_api && player_api.style)
+	{
+	    player_api.style.setProperty("display", "none", "important");
+	}
+    }
+
+    // Prevent next clip loading with AJAX on HTML5 player stream end.
+    if (!this.priority)
+    {
+	var video_el = player_api.getElementsByTagName("video");
+	if (video_el && video_el[0])
+	{
+	    var self = this;
+	    video_el[0].addEventListener("ended", function(){
+		self.sites["youtube.com"].player_stream_ended_action.apply(self,[]);
+	    }, true);
+	}
+    }
+
+    // FIX Displacement of the sidebar in playlists.
+    var watch7_sidebar = document.getElementById("watch7-sidebar");
+    if (watch7_sidebar)
+    {
+	watch7_sidebar.style.setProperty("margin-top", "-429px", "important");
     }
 
     if (document.getElementById("playnav-playview"))
@@ -570,20 +726,9 @@ function(object_data)
     }
 
     // Bug #33504 https://savannah.nongnu.org/bugs/?33504
-    object_data.parent.style.setProperty("overflow", "visible", "important");
-
-    var site_html5_player = 
-	this.find_site_html5_player_wrapper(object_data.parent);
-
-    if (site_html5_player)
+    if (object_data.parent)
     {
-	// The HTML5 player's controlls hide the Linterna Magica switch button.
-	site_html5_player.style.setProperty("margin-bottom", "30px", "important");
-
-	// The player is too close to YT visitors counter & buttons
-	object_data.parent.style.setProperty("margin-bottom",
-						 "50px", "important");
-	
+	object_data.parent.style.setProperty("overflow", "visible", "important");
     }
 
     // When YouTube's HTML5 player detects that the browser does not
@@ -617,36 +762,7 @@ function(object_data)
 	    // will not be displaced.
 	    html5_wrapper.style.setProperty('display', "none", "important");
 
-	    // If the HTML5 player has a fallback CSS class and LM can
-	    // access it it means there is no other option for
-	    // playback. We take over the player.
-	    //
-	    // See bug 37630
-	    // https://savannah.nongnu.org/bugs/index.php?37630
-	    if (this.priority.html5 > this.priority.self)
- 	    {
-		// It should be available, because it is used above in
-		// this function
-		var id = object_data.linterna_magica_id;
-
-		this.show_lm_interface(id);
-	    
-		// It should be available, because it is used above in
-		// this functon.
-		toggle_plugin = 
-		    document.getElementById("linterna-magica-"+
-					    "toggle-plugin-"+id);
-
-		var pn = toggle_plugin.parentNode.parentNode;
-		pn.removeChild(toggle_plugin.parentNode);
-
-		toggle_plugin_header =
-		    document.getElementById("linterna-magica-"+
-					    "toggle-plugin-header-"+id);
-		
-		toggle_plugin_header.parentNode.removeChild(toggle_plugin_header);
-	    }
-	    
+    
 	    // Hide site controls and video container. Should not
 	    // overlap with Linterna Magica now. Fixes the overlapping
 	    // of HTML5 player and LM.
@@ -666,6 +782,14 @@ function(object_data)
 	}
     }
 
+    var movie_player = document.getElementById("movie_player");
+    if (movie_player)
+    {
+	movie_player.style.setProperty("height",
+				       parseInt(object_data.outer_height)+2+
+				       "px", "important");
+    }
+
     // Without this the toggle plugin stays hidden below the title of
     // the clip
     var watch7 = document.getElementById('watch7-player');
@@ -674,14 +798,13 @@ function(object_data)
 	watch7.style.setProperty("height", 
 				 (parseInt(object_data.outer_height)+
 				  24)+"px", "important");
+    }
 
-	var movie_player = document.getElementById("movie_player");
-	if (movie_player)
-	{
-	    movie_player.style.setProperty("height", 
-					   parseInt(object_data.outer_height)+
-					   "px", "important");
-	}
+
+    var watch7_container = document.getElementById("watch7-container");
+    if (watch7_container)
+    {
+	watch7_container.style.setProperty("margin-top", "38px", "important");
     }
 
     return false;
